#! /bin/sh

# Deploys software for an instance. Does NOT initialize databases, and
# does NOT initialize datanodes

usage () {
    echo "usage: $0 [-only-config] inst_name" >&2
    exit 2
}

set -e
inst=""
only_config=0

while [ $# -gt 0 ]; do
    case "$1" in
	-only-config)
	    only_config=1; shift ;;
	-*)
	    usage ;;
	*)
	    if [ -z "$inst" ]; then
		inst="$1"
	    else
		usage
	    fi
	    shift
    esac
done

if [ -z "$inst" ]; then
    echo "Error: missing instance";
    usage
fi

if [ ! -d "instances/$inst" ]; then
    echo "Error: no such instance: $inst" >&2
    exit 2
fi

# Collect now all hosts from the .hosts files and form the union set:

hosts="$(awk '
  /^[:blank:]*$/ { next }
  /^[:blank:]*#/ { next }
  /^\w+[:blank:]*#/ { hosts[$1]=1; next }
  /^\w+[:blank:]*$/ { hosts[$1]=1; next }
  // { print "bad line in " FILENAME ": " $0 >"/dev/stderr"; next }
  END { for (host in hosts) { print host } }' \
    instances/$inst/*.hosts)"
if [ $? -ne 0 ]; then
    echo "Stopping after error" >&2
    exit 2
fi

. "instances/$inst/global.conf"
. "./inst.conf"

na="$(which netplex-admin)"
if [ -z "$na" ]; then
    echo "Error: netplex-admin not found in PATH"
fi

for host in $hosts; do
    echo "Deploying to $host:"
    ssh $host \
	"mkdir -p '$prefix/bin' '$prefix/etc' '$prefix/log' '$prefix/data'"
    scp instances/$inst/*.conf instances/$inst/*.hosts instances/$inst/*.sh  \
	$host:"$prefix/etc"
    scp $namenode_sql $host:"$prefix/etc"
    if [ $only_config -eq 0 ]; then
	scp $plasmad $nfs3d $plasma_datanode_init $plasma_admin "$na" \
	    $host:"$prefix/bin"
    fi
done


